import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React from 'react';
import { forEach } from 'lodash';
import { getElementProps } from './utils';
import { FormContext } from './form-context';

var Form = function (_React$Component) {
  _inherits(Form, _React$Component);

  function Form(props, context) {
    _classCallCheck(this, Form);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props, context));

    _this.onSubmit = function (e) {
      e.preventDefault();
      var isFormContextValid = _this.formContext.isValid();
      if (isFormContextValid) {
        var _this$props$onSubmit = _this.props.onSubmit,
            onSubmit = _this$props$onSubmit === undefined ? function () {} : _this$props$onSubmit;

        onSubmit(_this.formContext.fields.getValues());
      } else {
        _this.formContext.fields.updateFieldsAsUsed();
      }
    };

    _this.formContext = new FormContext();
    _this.formContext.setValidators(props.validators);
    _this.formContext.setValidatorsOptions(props.validatorsOptions);
    _this.formContext.setAsyncValidator(props.asyncValidator);
    _this.formContext.setAsyncValidatorOptions(props.asyncValidatorOptions);
    return _this;
  }

  Form.prototype.getChildContext = function getChildContext() {
    return { form: this.formContext };
  };

  Form.prototype.componentDidMount = function componentDidMount() {
    this.formContext.initialized = true;
    this.formContext.update();
  };

  Form.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    this.formContext.setValidators(nextProps.validators);
    this.formContext.setValidatorsOptions(nextProps.validatorsOptions);
    this.formContext.setAsyncValidator(nextProps.asyncValidator);
    this.formContext.setAsyncValidatorOptions(nextProps.asyncValidatorOptions);
    this.formContext.update();
  };

  Form.prototype.componentWillUnmount = function componentWillUnmount() {
    this.formContext.clear();
  };

  Form.prototype.reset = function reset() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.formContext.reset(value);
  };

  Form.prototype.render = function render() {
    var children = this.props.children;

    return React.createElement(
      'form',
      _extends({}, getElementProps(this.props), { onSubmit: this.onSubmit, noValidate: true }),
      children
    );
  };

  return Form;
}(React.Component);

Form.propTypes = {
  name: React.PropTypes.any,
  children: React.PropTypes.any,
  onSubmit: React.PropTypes.any
};
Form.childContextTypes = {
  form: React.PropTypes.instanceOf(FormContext).isRequired
};

var formConnector = function formConnector(component) {
  return function (c) {
    var WrapperComponent = function (_React$Component2) {
      _inherits(WrapperComponent, _React$Component2);

      function WrapperComponent() {
        _classCallCheck(this, WrapperComponent);

        return _possibleConstructorReturn(this, _React$Component2.apply(this, arguments));
      }

      WrapperComponent.prototype.componentDidMount = function componentDidMount() {
        var _this3 = this;

        this.fieldsSubscriberId = this.context.form.fields.subscribe(function () {
          _this3.forceUpdate();
        });
        this.validatorsSubscriberId = this.context.form.validators.subscribe(function () {
          _this3.forceUpdate();
        });
      };

      WrapperComponent.prototype.componentWillUnmount = function componentWillUnmount() {
        this.context.form.fields.removeSubscriber(this.fieldsSubscriberId);
        this.context.form.validators.removeSubscriber(this.validatorsSubscriberId);
        this.fieldsSubscriberId = null;
        this.validatorsSubscriberId = null;
      };

      WrapperComponent.prototype.render = function render() {
        var form = this.context.form;
        var props = {
          dirty: false,
          pristine: true,
          touched: false,
          isValid: form.isValid(),
          fields: form.fields.getFieldsOptions(),
          formErrors: form.validators.errors,
          fieldsErrors: form.fields.getErrors()
        };
        forEach(form.fields.getFieldsOptions(), function (field) {
          if (field.dirty) {
            props.dirty = true;
            props.pristine = false;
          }
          if (field.touched) {
            props.touched = true;
          }
        });
        return React.createElement(c, _extends({}, this.props, { form: props }));
      };

      return WrapperComponent;
    }(React.Component);

    WrapperComponent.contextTypes = {
      form: React.PropTypes.instanceOf(FormContext).isRequired
    };
    return WrapperComponent;
  }(component);
};

export { Form, formConnector };